//---------------------------------------------------------------------------


#pragma hdrstop

#include "Recorder.h"
#include "device/data.h"
#include "Utilities.h"

#include <json/json.h>
#include <fstream>
#include <time.h>
#include <stdio.h>
#include <assert.h>

//---------------------------------------------------------------------------

#pragma package(smart_init)


int Recorder::Store(const std::string &filename, const enum E_FMT format, const Data &data)
{
	int status;
	if (format == FMT_JSON)
		status = StoreJson(filename, data);
	else if (format == FMT_CSV)
		status = StoreCsv(filename, data);
	else
		return -1;
	return status;
}


int Recorder::StoreJson(const std::string &filename, const Data &data)
{
	Json::Value root;
	Json::StyledWriter writer;

	char* timeline = ctime( & ( data.timestamp.time ) );
	timeline[19]=0;
	char buf[64];
	sprintf(buf, "%15s.%03hu", &timeline[4], data.timestamp.millitm);

	root["Timestamp"] = buf;
	float fDivX;
	if (data.GetMode() == Data::MODE_RECORDER)
		fDivX = data.fRecorderInterval;
	else
		fDivX = data.fOscDivX;
	root["fDivX"] = fDivX;
	root["TriggerType"] = data.trigger_type;
	root["TriggerSource"] = data.trigger_source;	

	for (unsigned ch = 0; ch<data.GetChannelCount(); ch++)
	{
		root["Channels"][ch]["Coupling"] = data.getChannelConst(ch).iCouplingType;
		const Data::Channel& channel = data.getChannelConst(ch);
		const std::vector<float> *samples;
		if (data.GetMode() == Data::MODE_OSCILLOSCOPE || data.GetMode() == Data::MODE_SPECTRUM)
			samples = &channel.dataOsc;
		else
			samples = &channel.dataRec;

		for (unsigned int i=0; i<samples->size(); i++)
		{
			// see json_writer for default precision
			root["Channels"][ch]["Samples"][i] = (*samples)[i];
		}
	}

	std::string outputConfig = writer.write( root );
	std::ofstream ofs(filename.c_str());
	if (!ofs.is_open())
		return -2;

	ofs << outputConfig;
	ofs.close();
	return 0;
}

int Recorder::StoreCsv(const std::string &filename, const Data &data)
{
	std::ofstream ofs(filename.c_str());
	if (!ofs.is_open())
		return -2;

	unsigned int ch_count = data.GetChannelCount();
	if (ch_count == 0)
	{
		ofs.close();
		return -3;
    }

	ofs << "Time";
	for (unsigned int i=0; i<ch_count; i++)
	{
		ofs << ",Voltage " << GetChannelName(i).c_str();
	}

	float time = 0;


	const Data::Channel& channel = data.getChannelConst(0);
	const std::vector<float> *samples;
	if (data.GetMode() == Data::MODE_OSCILLOSCOPE || data.GetMode() == Data::MODE_SPECTRUM)
		samples = &channel.dataOsc;
	else
		samples = &channel.dataRec;
	unsigned int samples_cnt = samples->size();

	float fDivX;
	if (data.GetMode() == Data::MODE_RECORDER)
		fDivX = data.fRecorderInterval;
	else
		fDivX = data.fOscDivX;

	for (unsigned int i=0; i<samples_cnt; i++)
	{
		ofs << "\r\n" << time;
		time += fDivX;
		for (unsigned int ch=0; ch<ch_count; ++ch)
		{
			const Data::Channel& channel = data.getChannelConst(ch);
			const std::vector<float> *samples;
			if (data.GetMode() == Data::MODE_OSCILLOSCOPE || data.GetMode() == Data::MODE_SPECTRUM)
				samples = &channel.dataOsc;
			else
				samples = &channel.dataRec;

			ofs << "," << (*samples)[i];
		}
	}
	ofs.close();
	return 0;
}


